@@ -147,32 +147,42 @@ if "sphinx" in sys.modules:
 
 class ExampleDatabase(metaclass=_EDMeta):
     """
-    A Hypothesis database, for use in the |settings.database| setting.
+    A Hypothesis database, for use in |settings.database|.
 
-    Using an |ExampleDatabase| allows Hypothesis to
-    remember and replay failing examples, and ensure that your tests are never
-    flaky. We provide several concrete subclasses, and you can write a custom
-    database backed by your preferred way to store a ``dict[bytes, set[bytes]]``.
+    Hypothesis automatically saves failures to the database set in
+    |settings.database|. The next time the test is run, Hypothesis will replay
+    any failures from the database in |settings.database| for that test (in
+    |Phase.reuse|).
+
+    The database is best thought of as a cache that you never need to invalidate.
+    Entries may be transparently dropped when upgrading your Hypothesis version
+    or changing your test. Do not rely on the database for correctness; to ensure
+    Hypothesis always tries an input, use |@example|.
+
+    A Hypothesis database is a simple mapping of bytes to sets of bytes. Hypothesis
+    provides several concrete database subclasses. To write your own database class,
+    see :doc:`/how-to/custom-database`.
 
     Change listening
     ----------------
 
     An optional extension to |ExampleDatabase| is change listening. On databases
-    which support it, you can call |ExampleDatabase.add_listener| to add a function
-    as a change listener, which will be called whenever a value is added, deleted,
-    or moved. See |ExampleDatabase.add_listener| for details.
+    which support change listening, calling |ExampleDatabase.add_listener| adds
+    a function as a change listener, which will be called whenever a value is
+    added, deleted, or moved inside the database. See |ExampleDatabase.add_listener|
+    for details.
 
     All databases in Hypothesis support change listening. Custom database classes
-    are not required to support change listening, unless they want to be
-    compatible with features that require change listening. Currently, no Hypothesis
-    features require change listening.
+    are not required to support change listening, though they will not be compatible
+    with features that require change listening until they do so.
 
     .. note::
 
-        Change listening is required by `HypoFuzz <https://hypofuzz.com/>`_.
+        While no Hypothesis features currently require change listening, change
+        listening is required by `HypoFuzz <https://hypofuzz.com/>`_.
 
-    Methods
-    -------
+    Database methods
+    ----------------
 
     Required methods:
 
