@@ -26,7 +26,7 @@ from hypothesis.errors import HypothesisWarning
 if TYPE_CHECKING:
     from typing import TypeAlias
 
-    from hypothesis.internal.conjecture.data import ConjectureData
+    from hypothesis.internal.conjecture.data import ConjectureData, Status
 
 
 @dataclass
@@ -115,7 +115,9 @@ def make_testcase(
     coverage: Optional[dict[str, list[int]]] = None,
     phase: Optional[str] = None,
     backend_metadata: Optional[dict[str, Any]] = None,
-    status: Optional[TestCaseStatus] = None,  # overrides automatic calculation
+    status: Optional[
+        Union[TestCaseStatus, "Status"]
+    ] = None,  # overrides automatic calculation
     status_reason: Optional[str] = None,  # overrides automatic calculation
     # added to calculated metadata. If keys overlap, the value from this `metadata`
     # is used
@@ -140,6 +142,9 @@ def make_testcase(
         Status.INTERESTING: "failed",
     }
 
+    if status is not None and isinstance(status, Status):
+        status = status_map[status]
+
     return TestCaseObservation(
         type="test_case",
         status=status if status is not None else status_map[data.status],
